<?php

declare(strict_types = 1);

namespace Drupal\Tests\ldap_sso\Browser;

use Drupal\Core\Url;
use Drupal\ldap_servers\FakeBridge;
use Drupal\ldap_servers\FakeCollection;
use Drupal\ldap_servers\LdapUserAttributesInterface;
use Drupal\Tests\BrowserTestBase;
use Symfony\Component\Ldap\Entry;

/**
 * Test redirection behaviour with SSO enabled.
 *
 * @group ldap_sso
 */
class LdapSsoBrowserTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'user',
    'ldap_servers',
    'ldap_sso',
    'ldap_authentication',
    'ldap_user',
    'node',
    'path',
    'path_alias',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $_SERVER['REMOTE_USER'] = 'hpotter';

    $this->drupalCreateContentType([
      'type' => 'page',
    ]);

    // Create a node for front page.
    $node_front = $this->drupalCreateNode();

    // Create a node with a random alias.
    $this->nodeWithAlias = $this->drupalCreateNode([
      'type' => 'page',
      'status' => 1,
      'path' => '/' . $this->randomMachineName(),
    ]);

    // Configure 'node' as front page.
    $this->config('system.site')
      ->set('page.front', '/node/' . $node_front->id())
      ->save();

    /** @var \Drupal\Core\Entity\EntityTypeManager $manager */
    $manager = $this->container->get('entity_type.manager');
    $server = $manager->getStorage('ldap_server')->create([
      'id' => 'test',
      'timeout' => 30,
      'tls' => FALSE,
      'address' => 'example',
      'port' => 963,
      'basedn' => 'ou=people,dc=hogwarts,dc=edu',
      'user_attr' => 'cn',
      'unique_persistent_attr' => 'uid',
      'status' => TRUE,
      'mail_attr' => 'mail',
    ]);
    $server->save();
    $this->config('ldap_authentication.settings')
      ->set('sids', [$server->id()])
      // TODO: This should not be necessary, investigate schema.
      ->set('excludeIfTextInDn', [])
      ->set('allowOnlyIfTextInDn', [])
      ->save();
    $this->config('ldap_user.settings')
      ->set('acctCreation', LdapUserAttributesInterface::ACCOUNT_CREATION_LDAP_BEHAVIOUR)
      ->set('drupalAcctProvisionServer', $server->id())
      ->set('ldapUserSyncMappings', [
        'drupal' => [],
        'ldap' => [],
      ])
      ->set('drupalAcctProvisionTriggers', [
        LdapUserAttributesInterface::PROVISION_DRUPAL_USER_ON_USER_AUTHENTICATION,
        LdapUserAttributesInterface::PROVISION_DRUPAL_USER_ON_USER_ON_MANUAL_CREATION,
        LdapUserAttributesInterface::PROVISION_DRUPAL_USER_ON_USER_UPDATE_CREATE,
      ])
      ->save();
    $this->config('ldap_sso.settings')
      ->set('seamlessLogin', TRUE)
      // TODO: This should not be necessary, investigate schema / code.
      ->set('ssoExcludedHosts', [])
      ->set('ssoExcludedPaths', [])
      ->set('ssoSplitUserRealm', FALSE)
      ->set('ssoVariable', 'REMOTE_USER')
      ->save();
    $this->container->get('config.factory')->reset();
    $bridge = new FakeBridge(
      $this->container->get('logger.channel.ldap_servers'),
      $this->container->get('entity_type.manager')
    );
    $bridge->setServer($server);
    $ldap = $bridge->get();
    $collection = [
      '(cn=hpotter)' => new FakeCollection([
        new Entry(
          'cn=hpotter,ou=people,dc=hogwarts,dc=edu',
          [
            'cn' => ['hpotter'],
            'uid' => ['123'],
            'mail' => ['hpotter@example.com'],
          ],
        ),
      ]),
    ];
    $ldap->setQueryResult($collection);
  }

  /**
   * Test whether we are being redirected correctly.
   */
  public function testRedirection(): void {
    // Starting automatic redirection.
    $url = $this->buildUrl(Url::fromRoute('<front>'), ['absolute' => TRUE]);
    $session = $this->getSession();
    $session->getDriver()->getClient()->followRedirects(FALSE);
    $this->prepareRequest();
    $session->visit($url);
    self::assertEquals($url, $session->getCurrentUrl());
    self::assertEquals(302, $session->getStatusCode());
    self::assertEquals('/user/login/sso?destination=/node/1', $session->getResponseHeader('Location'));

    // Do not redirect on default path.
    $url = $this->buildUrl(Url::fromRoute('user.login'), ['absolute' => TRUE]);
    $session->reset();
    $session->getDriver()->getClient()->followRedirects(FALSE);
    $this->prepareRequest();
    $session->visit($url);
    self::assertEquals($url, $session->getCurrentUrl());
    self::assertEquals(200, $session->getStatusCode());

    // Do not redirect when we are currently logging in.
    $url = $this->buildUrl(Url::fromRoute('<front>'), ['absolute' => TRUE]);
    $session = $this->getSession();
    $session->reset();
    $session->getDriver()->getClient()->followRedirects(FALSE);
    $session->setCookie('sso_login_running', 'true');
    $this->prepareRequest();
    $session->visit($url);
    self::assertEquals($url, $session->getCurrentUrl());
    self::assertEquals(200, $session->getStatusCode());

    // Do not redirect when we have just logged out.
    $url = $this->buildUrl(Url::fromRoute('<front>'), ['absolute' => TRUE]);
    $session->reset();
    $session->getDriver()->getClient()->followRedirects(FALSE);
    $session->setCookie('sso_stop', 'true');
    $this->prepareRequest();
    $session->visit($url);
    self::assertEquals($url, $session->getCurrentUrl());
    self::assertEquals(200, $session->getStatusCode());
    self::assertFalse($session->getCookie('sso_login_running'));

    $url = $this->buildUrl(Url::fromRoute('ldap_sso.login_controller'), ['absolute' => TRUE]);
    $session->getDriver()->getClient()->followRedirects(TRUE);
    $session->setCookie('sso_login_running', 'true');
    $session->reset();
    $this->prepareRequest();
    $session->visit($url);
    $users = $this->container
      ->get('entity_type.manager')
      ->getStorage('user')
      ->loadByProperties(['mail' => 'hpotter@example.com']);
    self::assertTrue($session->getPage()->hasContent('You were not authenticated'));
    self::assertCount(0, $users);
    // TODO: Extend this test to actually log in successfully. Currently not
    // working since global variable seems unavailable.
  }

}
